<?php
/**
 * SPDX-FileCopyrightText: 2025 Sascha Nitsch (grumpydeveloper) https://contentnation.net/@grumpydevelop
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * @author Sascha Nitsch (grumpyveveloper)
 **/

namespace Federator\DIO;

/**
 * Do the Server2Server communication
 */
class Server
{
    /**
     * send activity to federated server
     *
     * @param \mysqli $dbh database handle
     * @param string $ourhost host url of our server (e.g. federator)
     * @param \Federator\Data\User $sender source user
     * @param \Federator\Data\FedUser $receiver federated user to receive the activity
     * @param \Federator\Data\ActivityPub\Common\Activity $activity activity to send
     * @return boolean true on success
     */
    public static function sendActivity($dbh, $ourhost, $sender, $receiver, $activity)
    {
        $receiverInboxUrl = $receiver->inboxURL;

        $json = json_encode($activity, JSON_UNESCAPED_SLASHES);

        if ($json === false) {
            throw new \Exception('Failed to encode JSON: ' . json_last_error_msg());
        }
        $digest = 'SHA-256=' . base64_encode(hash('sha256', $json, true));
        $date = gmdate('D, d M Y H:i:s') . ' GMT';
        echo "inboxurl $receiverInboxUrl\n";
        $parsedReceiverInboxUrl = parse_url($receiverInboxUrl);
        if ($parsedReceiverInboxUrl === false) {
            throw new \Exception('Failed to parse URL: ' . $receiverInboxUrl);
        }

        if (!isset($parsedReceiverInboxUrl['host']) || !isset($parsedReceiverInboxUrl['path'])) {
            throw new \Exception('Invalid inbox URL: missing host or path');
        }
        $extHost = $parsedReceiverInboxUrl['host'];
        $path = $parsedReceiverInboxUrl['path'];

        // Build the signature string
        $signatureString = "(request-target): post {$path}\n" .
            "host: {$extHost}\n" .
            "date: {$date}\n" .
            "digest: {$digest}";

        // Get rsa private key
        $privateKey = \Federator\DIO\User::getrsaprivate($dbh, $sender->id); // OR from DB
        if ($privateKey === false) {
            throw new \Exception('Failed to get private key');
        }
        $pkeyId = openssl_pkey_get_private($privateKey);

        if ($pkeyId === false) {
            throw new \Exception('Invalid private key');
        }
echo "signaturestring $signatureString\n";
        openssl_sign($signatureString, $signature, $pkeyId, OPENSSL_ALGO_SHA256);
        $signature_b64 = base64_encode($signature);

        // Build keyId (public key ID from your actor object)
        $keyId = $ourhost . '/' . $sender->id . '#main-key';

        $signatureHeader = 'keyId="' . $keyId
            . '",algorithm="rsa-sha256",headers="(request-target) host date digest",signature="' . $signature_b64 . '"';

        $ch = curl_init($receiverInboxUrl);
        if ($ch === false) {
            throw new \Exception('Failed to initialize cURL');
        }
        $headers = [
            'Host: ' . $extHost,
            'Date: ' . $date,
            'Digest: ' . $digest,
            'Content-Type: application/activity+json',
            'Signature: ' . $signatureHeader,
            'Accept: application/activity+json',
        ];
print_r($headers);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        $response = curl_exec($ch);
        curl_close($ch);

        if ($response === false) {
            throw new \Exception('Failed to send activity: ' . curl_error($ch));
        } else {
            $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            if ($httpcode != 200 && $httpcode != 202) {
                throw new \Exception('Unexpected HTTP code ' . $httpcode . ':' . $response);
            }
        }
        if ($response !== true) {
            error_log($response);
        }
        return true;
    }
}
